<?php

function _pp_update_6006() {
  $ret = array();
  
  // Enable new module pp_ig. Also do the database table install
  // This should also create the vocabulary for image galleries
  if (!module_exists('pp_ig')) {
    module_enable(array('pp_ig'));
    if (drupal_get_installed_schema_version('pp_ig') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('pp_ig'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module ProsePoint Image Gallery.')); 
  }

  // Adjust taxonomy weight for ifimage type
  $content_extra_weights_ifimage = variable_get('content_extra_weights_ifimage', array('body_field' => 0, 'menu' => 1));
  $content_extra_weights_ifimage['taxonomy'] = 1;
  variable_set('content_extra_weights_ifimage', $content_extra_weights_ifimage);

  // Fill in new permissions: 
  require_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.permissions.inc');

  // Add 'access pp image galleries' to anon and auth
  prosepoint_add_permission(1, 'access pp image galleries');
  prosepoint_add_permission(2, 'access pp image galleries');
  $ret[] = array('success' => TRUE, 'query' => t('Added permission access pp image galleries to anonymous and authenticated users.')); 

  // Add 'administer pp image galleries' to editors
  prosepoint_add_permission(4, 'administer pp image galleries');
  $ret[] = array('success' => TRUE, 'query' => t('Added permission administer pp image galleries to editor users.')); 

  // Add 'administer pp image galleries' to administrators
  $admin_rid = prosepoint_get_admin_rid();
  if ($admin_rid) {
    prosepoint_add_permission($admin_rid, 'administer pp image galleries');
    $ret[] = array('success' => TRUE, 'query' => t('Added permission administer pp image galleries to administrator users.')); 
  }
  else {
    drupal_set_message(t('Unable to locate administrator role to patch permissions. Please contact www.prosepoint.org for further help.'), 'warning');
  }

  // Add new variable: 'pp_ig_gallery_thumbnail_preset'
  variable_set('pp_ig_gallery_thumbnail_preset', 'square');

  // Direct pathauto not to create aliases for image galleries
  $vocab_id = variable_get('pp_ig_gallery_vocabulary', FALSE);
  if ($vocab_id) {
    variable_set('pathauto_taxonomy_' . $vocab_id . '_pattern', 'image_gallery/[tid]');
  }
  
  return $ret;
}


function _pp_update_6007() {
  $ret = array();

  // Fill in variable pp_show_story_list_for_source
  variable_set('pp_show_story_list_for_source', 1);

  // Enable module poormanscron
  if (!module_exists('poormanscron')) {
    module_enable(array('poormanscron'));
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module Poorsmancron.')); 
  }

  return $ret;
}


function _pp_update_6008() {
  $ret = array();

  // Make pp_ch_channel_vocabulary handled by pp_ch.module
  $vocab = taxonomy_vocabulary_load(variable_get('pp_ch_channel_vocabulary', '--'));
  if ($vocab) {
    $vocab->module = 'pp_ch';
    $vocab = (array) $vocab;
    taxonomy_save_vocabulary($vocab);
  }

  // Adjust menu paths and url aliases for story listing channels.
  // Move path from taxonomy/term/N to channel/N/stories.
  $channels = pp_ch_get_channels();
  foreach ($channels as $term) {
    $channel = pp_ch_load($term->tid);
    if (empty($channel['editions_enable'])) {
      $oldpath = 'taxonomy/term/' . $term->tid;
      $newpath = 'channel/' . $term->tid . '/stories';
      
      $alias = drupal_get_path_alias($oldpath, '');
      if ($oldpath != $alias) {
        // Change alias from oldpath to newpath
        path_set_alias($oldpath, null);
        path_set_alias($newpath, $alias);
      }

      $menu_name = variable_get('menu_default_node_menu', 'primary-links');  
      $mlid = db_result(db_query_range("SELECT mlid FROM {menu_links} WHERE link_path = '%s' AND menu_name = '%s' AND module = 'menu' ORDER BY mlid ASC", $oldpath, $menu_name, 0, 1));
      if (!$mlid) {
        $mlid = db_result(db_query_range("SELECT mlid FROM {menu_links} WHERE link_path = '%s' AND module = 'menu' ORDER BY mlid ASC", $oldpath, 0, 1));
      }
      if ($mlid) {
        $m = menu_link_load($mlid);
        if ($m) {
          // Change link_path from oldpath to newpath
          $m['link_path'] = $newpath;
          menu_link_save($m);
        }
      }

      // If the frontpage was pointing to oldpath, now point it to newpath
      $frontpage = variable_get('site_frontpage', 'node');
      if ($frontpage == $oldpath) {
        variable_set('site_frontpage', $newpath);
      }
    }
  }
  $ret[] = array('success' => TRUE, 'query' => t('Adjusted menu items and aliases for story listing channels.')); 

  return $ret;
}

function _pp_update_6009() {
  $ret = array();

  // Add block_display column to table term_channels
  db_add_field($ret, 'term_channels', 'block_display', array('type' => 'text', 'size' => 'medium', 'not null' => TRUE, 'serialize' => TRUE, 'initial' => serialize(array('mode' => 'list', 'view' => 'title', 'count' => 5))));

  // Fill in new block_display settings
  $terms = pp_ch_get_channels();
  foreach ($terms as $term) {
    $channel = pp_ch_load($term->tid);
    if ($channel['editions_enable']) {
      // For this scheduled editions channel, change the block_display parameters to match
      db_query("UPDATE {term_channels} SET block_display = '%s' WHERE tid = %d", serialize(array('mode' => 'schedule', 'view' => 'teaser', 'count' => 5)), $term->tid);
    }
  }

  // Enable pp_cs.module
  if (!module_exists('pp_cs')) {
    module_enable(array('pp_cs'));
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module %module.', array('%module' => 'pp_cs')));
  }

  return $ret;  
}

function _pp_update_6010() {
  $ret = array();

  // Check for admin role and warn and abort if it is not the expected value
  if (!pp_check_admin_rid()) {
    $ret['#abort'] = array('success' => TRUE, 'query' => t('Update aborted.'));
    return $ret;
  }
  
  // Enable workflow.module
  if (!module_exists('workflow')) {
    module_enable(array('workflow'));
    if (drupal_get_installed_schema_version('workflow') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('workflow'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module Workflow.')); 
  }

  // Enable workflow_access.module
  if (!module_exists('workflow_access')) {
    module_enable(array('workflow_access'));
    if (drupal_get_installed_schema_version('workflow_access') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('workflow_access'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module Workflow Access.')); 
  }

  // Enable workflow_state.module
  if (!module_exists('workflow_state')) {
    module_enable(array('workflow_state'));
    if (drupal_get_installed_schema_version('workflow_state') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('workflow_state'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module Workflow State View.')); 
  }

  // Refresh database schema
  drupal_get_schema(null, TRUE);

  // Initialise workflow configuration
  // Note: If we are doing a full configuration instead of an incremental update,
  //    we don't have to worry about past history. 
  require_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.workflow.inc');
  prosepoint_config_workflow();
  $ret[] = array('success' => TRUE, 'query' => t('Created new workflow.')); 

  return $ret;
}

function _pp_update_6011() {
  $ret = array();

  // Migrate data from field_extended_status to workflow state
  $workflow_states = variable_get('pp_publishing_states', FALSE);
  $extended_status_workflow_state_map = array(
    0 => $workflow_states['draft']['sid'],    // Unpublished doesn't have a direct counterpart
    1 => $workflow_states['public']['sid'],
    2 => $workflow_states['draft']['sid'],
    3 => $workflow_states['review']['sid'],
    4 => $workflow_states['archived']['sid'],
  );
  
  $count = 0;  
  $quanta = 100;
  // Find $quanta nodes to process, using -1 as a sentinel value
  $result = db_query("SELECT DISTINCT(node.nid) FROM {node} node LEFT JOIN {content_type_story} content_type_story ON node.vid = content_type_story.vid WHERE node.type = '%s' AND content_type_story.field_extended_status_value <> %d ORDER BY node.nid LIMIT %d", 'story', -1, $quanta);
  while ($object = db_fetch_object($result)) {
    $node = node_load($object->nid);
    $extended_status = $node->field_extended_status[0]['value'];
    // We have to force this through because workflow.module may have some outdated data due to caching
    workflow_execute_transition($node, $extended_status_workflow_state_map[$extended_status], NULL, TRUE);
    
    // Poke -1 into field_extended_status to mark this node has been done
    db_query("UPDATE {content_type_story} SET field_extended_status_value = %d WHERE vid = %d", -1, $node->vid);
    $count++;
  }
  if ($count) {
    $ret[] = array('success' => TRUE, 'query' => t('Migrated !num stories to new workflow states.', array('!num' => $count)));
    // If we updated any nodes, then bail out and run this update again.
    pp_update_run_again();
    return $ret;
  }

  // Add workflow permissions
  require_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.permissions.inc');
  // staff = 3
  prosepoint_add_permission(3, array('access workflow summary views', 'schedule workflow transitions'));
  // editor = 4
  prosepoint_add_permission(4, array('access workflow summary views', 'schedule workflow transitions'));
  // administrator = 5
  prosepoint_add_permission(5, array('access workflow summary views', 'schedule workflow transitions', 'administer workflow'));

  // Remove story.field_extended_status
  module_load_include('inc', 'content', 'includes/content.crud');
  content_field_instance_delete('field_extended_status', 'story');
  $ret[] = array('success' => TRUE, 'query' => t('Removed field_extended_status.')); 

  // Patch edition.field_headlines and edition.field_stories to remove views argument
  $edition_fields = content_field_instance_read(array('type_name' => 'edition'), TRUE);

  foreach ($edition_fields as $field) {
    if ($field['field_name'] == 'field_headlines' || $field['field_name'] == 'field_stories') {
      $field['advanced_view_args'] = '';
      content_field_instance_update($field);
    }
  }
  $ret[] = array('success' => TRUE, 'query' => t('Patched field_headlines and field_stories to remove views argument.')); 

  // Trigger node access rebuild
  node_access_needs_rebuild(TRUE);

  // Adjust weights for workflow form and pp_ed_img_extract_preset form item, for all applicable types
  $content_extra_weights_story = variable_get('content_extra_weights_story', array(
    'title' => -9,
    'taxonomy' => -4, 
    'body_field' => 0,
    'menu' => 2,
  ));
  $content_extra_weights_story['pp_ed_img_extract_preset'] = 1;
  $content_extra_weights_story['workflow'] = -4;
  variable_set('content_extra_weights_story', $content_extra_weights_story);

  // Enable pp_img_extract.module
  if (!module_exists('pp_img_extract')) {
    module_enable(array('pp_img_extract'));
    if (drupal_get_installed_schema_version('pp_img_extract') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('pp_img_extract'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module ProsePoint image extract.')); 
  }

  // Upgrade settings for teaser image extraction
  $pp_ed_img_extract_types = variable_get('pp_ed_img_extract_types', FALSE);
  $pp_ed_img_extract_preset = variable_get('pp_ed_img_extract_preset', FALSE);
  $pp_ed_img_extract_weight = variable_get('pp_ed_img_extract_weight', FALSE);

  $pp_img_extract_settings = array('weight' => $pp_ed_img_extract_weight);
  foreach ($pp_ed_img_extract_types as $type) {
    if ($type) {
      $pp_img_extract_settings['presets'][$type] = $pp_ed_img_extract_preset;
    }
  }
  
  variable_set('pp_img_extract_settings', $pp_img_extract_settings);
  variable_del('pp_ed_img_extract_types');
  variable_del('pp_ed_img_extract_preset');
  variable_del('pp_ed_img_extract_weight');
  $ret[] = array('success' => TRUE, 'query' => t('Upgraded teaser image extraction settings.')); 

  // Sites installed with ProsePoint 0.18, 0.19, 0.20 have a botched source.field_photo definition.
  // Need to delete this and replace the field with the correct settings. 
  $result = content_field_instance_read(array('type_name' => 'source', 'field_name' => 'field_photo'), TRUE);
  if (!empty($result)) {
    // There should only be one returned result
    $current_field = array_shift($result);
    if ($current_field['field_name'] == 'field_photo' && $current_field['type'] == 'image' && $current_field['module'] == 'imagefield') {
      // Yep, this field has incorrect settings. 
      include_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.content.inc');
      $source_type = prosepoint_unserialize_complex_type('source');

      // Search for our field_photo definition just in case it's not the first one
      foreach ($source_type['fields'] as $field) {
        if ($field['field_name'] == 'field_photo') {
          $type_name = 'source';
          $type_label = node_get_types('name', $type_name);
          $field_name = 'field_photo';
          // Might need to overwrite the content type name if a new type was created.
          $field['type_name'] = $type_name;
          
          // Delete the existing field AND all instances
          // We handle this ourselves because the field is inactive and this causes problems
          //   for content_field_XXX(). 
          // We directly edit the database, and also skip the content_alter_schema()
          //   which content_field_instance_delete() would have done. If the field is
          //   inactive from the beginning, there should be no schema to alter anyway. One hopes ... 
          db_query("DELETE FROM {". content_instance_tablename() .
            "} WHERE field_name = '%s' AND type_name = '%s'", $field['field_name'], $field['type_name']);
          db_query("DELETE FROM {". content_field_tablename() ."} WHERE field_name = '%s'", $field['field_name']);
          content_clear_type_cache(TRUE);
          
          $field = content_field_instance_create($field);
          $message = t('The field %field_label (%field_name) was updated for the content type %type.', array(
            '%field_label' => $field['widget']['label'], '%field_name' => $field_name, '%type' => $type_label));
          $ret[] = array('success' => TRUE, 'query' => $message);
          break;
        }
      }
    }
  }
  
  // Enable pp_anr.module
  if (!module_exists('pp_anr')) {
    module_enable(array('pp_anr'));
    if (drupal_get_installed_schema_version('pp_anr') == SCHEMA_UNINSTALLED) {
      drupal_install_modules(array('pp_anr'));
    }
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module ProsePoint nodereference.')); 
  }

  // Convert field_headlines to use ProsePoint nodereference (pp_anr)
  $fields = content_field_instance_read(array('field_name' => 'field_headlines'));
  foreach ($fields as $field) {
    $field['type'] = 'pp_anr';
    $field['module'] = 'pp_anr';
    $field['widget']['type'] = 'pp_anr_autocomplete';
    $field['widget']['module'] = 'pp_anr';
    $field['widget']['default_value'] = array(0 => array('nid' => NULL, 'format' => 'teaser'));
    $field['display_settings']['full']['format'] = 'defer';
    $field['referenceable_types']['story'] = FALSE;
    content_field_instance_update($field);
  }
  $ret[] = array('success' => TRUE, 'query' => t('Converted field_headlines to use ProsePoint nodereference.')); 

  // Initialise all field_headlines_format values to 'teaser'
  $db_info = content_database_info(content_fields('field_headlines'));
  db_query("UPDATE {" . $db_info['table'] . "} SET field_headlines_format = 'teaser'");
  
  return $ret;
}

function _pp_update_6012() {
  $ret = array();

  // Check date.module's schema version and abort if date_update_6005() hasn't yet been run
  $date_schema_version = db_result(db_query("SELECT schema_version FROM {system} WHERE type = '%s' AND name = '%s'", 'module', 'date'));
  if ($date_schema_version < 6005) {
    drupal_set_message(t('Some updates are still pending. Please return to <a href="@update-php">update.php</a> and run the remaining updates.', array('@update-php' => base_path() .'update.php?op=selection')), 'error', FALSE);
    $ret['#abort'] = array('success' => FALSE, 'query' => t('Some updates are still pending.<br/>Please re-run the update script.'));
    return $ret;
  }

  // After date_update_6005(), we clean up the many (extraneous?) date format types and date formats it creates
  // Delete relevant date_format_* variables first
  db_query("DELETE FROM {variable} WHERE name LIKE '%s'", 'date_format_field_%%');
  variable_del('date_format_time');
  variable_del('date_format_time_timezone');
  variable_del('date_format_ical');
  variable_del('date_format_timestamp');
  variable_del('date_format_feed');
  variable_del('date_format_iso');
  
  // Delete all user defined date formats and date format types
  $ret[] = update_sql("DELETE FROM {date_formats}");
  $ret[] = update_sql("DELETE FROM {date_format_types} WHERE locked = 0");

  // Add a story_date date format and date format type
  // Add custom format 'l, F jS, Y'
  $format = array();
  $format['format'] = 'l, F jS, Y';
  $format['type'] = 'custom';
  $format['locked'] = 0;
  $format['is_new'] = 1;
  date_format_save($format);

  // Add new date format type
  $format_type = array();
  $format_type['title'] = t('Story Date');
  $format_type['type'] = 'story_date';
  $format_type['locked'] = 1;
  $format_type['is_new'] = 1;
  date_format_type_save($format_type);
  variable_set('date_format_story_date', $format['format']);

  // Point field_date (of story type) to display as story_date
  module_load_include('inc', 'content', 'includes/content.crud');
  $fields = content_field_instance_read(array('field_name' => 'field_date'));
  foreach ($fields as $field) {
    $field['default_format'] = 'story_date';
    $field['display_settings'][4]['format'] = 'default';
    $field['display_settings'][2]['format'] = 'default';
    $field['display_settings'][3]['format'] = 'default';
    $field['display_settings']['teaser']['format'] = 'default';
    $field['display_settings']['full']['format'] = 'default';
    content_field_instance_update($field);
  }
  $ret[] = array('success' => TRUE, 'query' => t('Converted field_date to use date format story_date.')); 

  // Add permission 'view date repeats' to admin, editor, staff
  require_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.permissions.inc');
  // staff = 3
  prosepoint_add_permission(3, array('view date repeats'));
  // editor = 4
  prosepoint_add_permission(4, array('view date repeats'));
  // administrator = 5
  prosepoint_add_permission(5, array('view date repeats'));

  // Revert view 'tracker' if it still contains an out of date argument handler 'uid'
  $view = views_get_view('tracker');
  if (isset($view->display['default']->display_options['arguments']['uid'])) {
    $view->delete();
    $ret[] = array('success' => TRUE, 'query' => t('Reverted view %view.', array('%view' => 'tracker'))); 
  }
  
  return $ret;
}

function _pp_update_6013() {
  $ret = array();

  // Add 'data' column to term_channels table
  $fields = array(
    'data' => array('type' => 'text', 'size' => 'medium', 'not null' => TRUE, 'serialize' => TRUE),
  );
  db_add_field($ret, 'term_channels', 'data', $fields['data']);

  // Initialise 'data' column
  db_query("UPDATE {term_channels} SET data = '%s'", serialize(array('title_on_channel_page' => 1, 'title_on_listing_page' => 1)));

  return $ret;
}

function _pp_update_6014() {
  $ret = array();

  // Enable new module general_services.module
  if (!module_exists('general_services')) {
    module_enable(array('general_services'));
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module General Services.')); 
  }

  // Migrate service_links_show_* variables into new format
  if (!variable_set('service_links_show', NULL)) {
    $settings = array();
    $result = db_query("SELECT name, value FROM {variable} WHERE name LIKE 'service_links_show_%'");
    while ($object = db_fetch_object($result)) {
      $object->value = unserialize($object->value);
      if (!empty($object->value)) {
        $settings[substr($object->name, 19)] = 1;
      }
    }
    variable_set('service_links_show', $settings);
    $ret[] = array('success' => TRUE, 'query' => t('Migrated service links settings to new format.'));
  }

  // Delete service_links_show_* variables
  db_query("DELETE FROM {variable} WHERE name LIKE 'service_links_show_%'");

  // Enable new module jquery_ui
  if (!module_exists('jquery_ui')) {
    module_enable(array('jquery_ui'));
    $ret[] = array('success' => TRUE, 'query' => t('Enabled module jQuery UI.')); 
  }

  return $ret;
}
